@@ -46,7 +46,7 @@ class DomaPanDAWork(Work):
                  num_retries=5,
                  task_log=None,
                  task_cloud=None,
-                 task_rss=0):
+                 task_rss=1000):
 
         super(DomaPanDAWork, self).__init__(executable=executable, arguments=arguments,
                                             parameters=parameters, setup=setup, work_type=TransformType.Processing,
@@ -61,6 +61,9 @@ class DomaPanDAWork(Work):
         self.panda_url = None
         self.panda_url_ssl = None
         self.panda_monitor = None
+        self.panda_auth = None
+        self.panda_auth_vo = None
+        self.panda_config_root = None
 
         self.dependency_map = dependency_map
         self.dependency_map_deleted = []
@@ -86,6 +89,8 @@ class DomaPanDAWork(Work):
         self.retry_number = 0
         self.num_retries = num_retries
 
+        self.poll_panda_jobs_chunk_size = 2000
+
         self.load_panda_urls()
 
     def my_condition(self):
@@ -128,6 +133,15 @@ class DomaPanDAWork(Work):
                 self.panda_url_ssl = panda_config.get('panda', 'panda_url_ssl')
                 os.environ['PANDA_URL_SSL'] = self.panda_url_ssl
                 # self.logger.debug("Panda url ssl: %s" % str(self.panda_url_ssl))
+            if panda_config.has_option('panda', 'panda_auth'):
+                self.panda_auth = panda_config.get('panda', 'panda_auth')
+                os.environ['PANDA_AUTH'] = self.panda_auth
+            if panda_config.has_option('panda', 'panda_auth_vo'):
+                self.panda_auth_vo = panda_config.get('panda', 'panda_auth_vo')
+                os.environ['PANDA_AUTH_VO'] = self.panda_auth_vo
+            if panda_config.has_option('panda', 'panda_config_root'):
+                self.panda_config_root = panda_config.get('panda', 'panda_config_root')
+                os.environ['PANDA_CONFIG_ROOT'] = self.panda_config_root
 
         if not self.panda_monitor and 'PANDA_MONITOR_URL' in os.environ and os.environ['PANDA_MONITOR_URL']:
             self.panda_monitor = os.environ['PANDA_MONITOR_URL']
@@ -138,6 +152,12 @@ class DomaPanDAWork(Work):
         if not self.panda_url_ssl and 'PANDA_URL_SSL' in os.environ and os.environ['PANDA_URL_SSL']:
             self.panda_url_ssl = os.environ['PANDA_URL_SSL']
             # self.logger.debug("Panda url ssl: %s" % str(self.panda_url_ssl))
+        if not self.panda_auth and 'PANDA_AUTH' in os.environ and os.environ['PANDA_AUTH']:
+            self.panda_auth = os.environ['PANDA_AUTH']
+        if not self.panda_auth_vo and 'PANDA_AUTH_VO' in os.environ and os.environ['PANDA_AUTH_VO']:
+            self.panda_auth_vo = os.environ['PANDA_AUTH_VO']
+        if not self.panda_config_root and 'PANDA_CONFIG_ROOT' in os.environ and os.environ['PANDA_CONFIG_ROOT']:
+            self.panda_config_root = os.environ['PANDA_CONFIG_ROOT']
 
     def set_agent_attributes(self, attrs, req_attributes=None):
         if 'life_time' not in attrs[self.class_name] or int(attrs[self.class_name]['life_time']) <= 0:
@@ -145,6 +165,8 @@ class DomaPanDAWork(Work):
         super(DomaPanDAWork, self).set_agent_attributes(attrs)
         if 'num_retries' in self.agent_attributes and self.agent_attributes['num_retries']:
             self.num_retries = int(self.agent_attributes['num_retries'])
+        if 'poll_panda_jobs_chunk_size' in self.agent_attributes and self.agent_attributes['poll_panda_jobs_chunk_size']:
+            self.poll_panda_jobs_chunk_size = int(self.agent_attributes['poll_panda_jobs_chunk_size'])
 
     def depend_on(self, work):
         for job in self.dependency_map:
@@ -408,11 +430,11 @@ class DomaPanDAWork(Work):
 
     def submit_panda_task(self, processing):
         try:
-            from pandatools import Client
+            from pandaclient import Client
 
             proc = processing['processing_metadata']['processing']
             task_param = proc.processing_metadata['task_param']
-            return_code = Client.insertTaskParams(task_param, verbose=True)
+            return_code = Client.insertTaskParams(task_param, verbose=False)
             if return_code[0] == 0:
                 return return_code[1][1]
             else:
@@ -440,7 +462,7 @@ class DomaPanDAWork(Work):
                 proc.submitted_at = datetime.datetime.utcnow()
 
     def get_panda_task_id(self, processing):
-        from pandatools import Client
+        from pandaclient import Client
 
         start_time = datetime.datetime.utcnow() - datetime.timedelta(hours=10)
         start_time = start_time.strftime('%Y-%m-%d %H:%M:%S')
@@ -465,7 +487,7 @@ class DomaPanDAWork(Work):
 
     def poll_panda_task_status(self, processing):
         if 'processing' in processing['processing_metadata']:
-            from pandatools import Client
+            from pandaclient import Client
 
             proc = processing['processing_metadata']['processing']
             status, task_status = Client.getTaskStatus(proc.workload_id)
@@ -618,8 +640,8 @@ class DomaPanDAWork(Work):
         return update_contents
 
     def map_panda_ids(self, unregistered_job_ids, input_output_maps):
-        self.logger.debug("map_panda_ids, unregistered_job_ids: %s" % str(unregistered_job_ids))
-        from pandatools import Client
+        self.logger.debug("map_panda_ids, unregistered_job_ids[:10]: %s" % str(unregistered_job_ids[:10]))
+        from pandaclient import Client
 
         # updated_map_ids = []
         full_update_contents = []
@@ -646,8 +668,8 @@ class DomaPanDAWork(Work):
         return full_update_contents
 
     def get_status_changed_contents(self, unterminated_job_ids, input_output_maps, panda_id_to_map_ids):
-        self.logger.debug("get_status_changed_contents, unterminated_job_ids: %s" % str(unterminated_job_ids))
-        from pandatools import Client
+        self.logger.debug("get_status_changed_contents, unterminated_job_ids[:10]: %s" % str(unterminated_job_ids[:10]))
+        from pandaclient import Client
 
         full_update_contents = []
         chunksize = 2000
@@ -673,10 +695,10 @@ class DomaPanDAWork(Work):
 
         return update_contents
 
-    def poll_panda_task(self, processing=None, input_output_maps=None):
+    def poll_panda_task_old(self, processing=None, input_output_maps=None):
         task_id = None
         try:
-            from pandatools import Client
+            from pandaclient import Client
 
             jobs_ids = None
             if processing:
@@ -687,8 +709,9 @@ class DomaPanDAWork(Work):
 
                 if task_id:
                     # ret_ids = Client.getPandaIDsWithTaskID(task_id, verbose=False)
+                    self.logger.debug("poll_panda_task, task_id: %s" % str(task_id))
                     task_info = Client.getJediTaskDetails({'jediTaskID': task_id}, True, True, verbose=False)
-                    self.logger.info("poll_panda_task, task_info: %s" % str(task_info))
+                    self.logger.debug("poll_panda_task, task_info[0]: %s" % str(task_info[0]))
                     if task_info[0] != 0:
                         self.logger.warn("poll_panda_task %s, error getting task status, task_info: %s" % (task_id, str(task_info)))
                         return ProcessingStatus.Submitting, {}
@@ -743,10 +766,212 @@ class DomaPanDAWork(Work):
             # raise exceptions.IDDSException(msg)
         return ProcessingStatus.Submitting, []
 
+    def poll_panda_jobs(self, job_ids):
+        job_ids = list(job_ids)
+        self.logger.debug("poll_panda_jobs, poll_panda_jobs_chunk_size: %s, job_ids[:10]: %s" % (self.poll_panda_jobs_chunk_size, str(job_ids[:10])))
+        from pandaclient import Client
+
+        # updated_map_ids = []
+        inputname_jobid_map = {}
+        chunksize = self.poll_panda_jobs_chunk_size
+        chunks = [job_ids[i:i + chunksize] for i in range(0, len(job_ids), chunksize)]
+        for chunk in chunks:
+            jobs_list = Client.getJobStatus(chunk, verbose=0)[1]
+            if jobs_list:
+                self.logger.debug("poll_panda_jobs, input jobs: %s, output_jobs: %s" % (len(chunk), len(jobs_list)))
+                for job_info in jobs_list:
+                    job_status = self.get_content_status_from_panda_status(job_info)
+                    if job_info and job_info.Files and len(job_info.Files) > 0:
+                        for job_file in job_info.Files:
+                            # if job_file.type in ['log']:
+                            if job_file.type not in ['pseudo_input']:
+                                continue
+                            if ':' in job_file.lfn:
+                                pos = job_file.lfn.find(":")
+                                input_file = job_file.lfn[pos + 1:]
+                                # input_file = job_file.lfn.split(':')[1]
+                            else:
+                                input_file = job_file.lfn
+                            inputname_jobid_map[input_file] = {'panda_id': job_info.PandaID, 'status': job_status}
+            else:
+                self.logger.warn("poll_panda_jobs, input jobs: %s, output_jobs: %s" % (len(chunk), jobs_list))
+        return inputname_jobid_map
+
+    def get_job_maps(self, input_output_maps):
+        inputname_mapid_map = {}
+        finished_jobs, failed_jobs = [], []
+        for map_id in input_output_maps:
+            inputs = input_output_maps[map_id]['inputs']
+            outputs = input_output_maps[map_id]['outputs']
+            outputs_short = []
+            for content in outputs:
+                outputs_short.append({'content_id': content['content_id'],
+                                      'status': content['status'],
+                                      'substatus': content['substatus'],
+                                      'content_metadata': content['content_metadata']})
+
+                if content['status'] in [ContentStatus.Available]:
+                    if 'panda_id' in content['content_metadata']:
+                        finished_jobs.append(content['content_metadata']['panda_id'])
+                elif content['status'] in [ContentStatus.Failed, ContentStatus.FinalFailed,
+                                           ContentStatus.Lost, ContentStatus.Deleted,
+                                           ContentStatus.Missing]:
+                    if 'panda_id' in content['content_metadata']:
+                        failed_jobs.append(content['content_metadata']['panda_id'])
+            for content in inputs:
+                inputname_mapid_map[content['name']] = {'map_id': map_id,
+                                                        'outputs': outputs_short}
+        return finished_jobs + failed_jobs, inputname_mapid_map
+
+    def get_update_contents(self, inputnames, inputname_mapid_map, inputname_jobid_map):
+        self.logger.debug("get_update_contents, inputnames[:5]: %s" % str(inputnames[:5]))
+        self.logger.debug("get_update_contents, inputname_mapid_map[:5]: %s" % str({k: inputname_mapid_map[k] for k in inputnames[:5]}))
+        self.logger.debug("get_update_contents, inputname_jobid_map[:5]: %s" % str({k: inputname_jobid_map[k] for k in inputnames[:5]}))
+
+        update_contents = []
+        num_updated_contents, num_unupdated_contents = 0, 0
+        for inputname in inputnames:
+            panda_id_status = inputname_jobid_map[inputname]
+            panda_id = panda_id_status['panda_id']
+            panda_status = panda_id_status['status']
+            map_id_contents = inputname_mapid_map[inputname]
+            contents = map_id_contents['outputs']
+            for content in contents:
+                if content['substatus'] != panda_status:
+                    if 'panda_id' in content['content_metadata'] and content['content_metadata']['panda_id']:
+                        # if content['content_metadata']['panda_id'] != job_info.PandaID:
+                        if content['content_metadata']['panda_id'] < panda_id:
+                            # new panda id is the bigger one.
+                            if 'old_panda_id' not in content['content_metadata']:
+                                content['content_metadata']['old_panda_id'] = []
+                            if content['content_metadata']['panda_id'] not in content['content_metadata']['old_panda_id']:
+                                content['content_metadata']['old_panda_id'].append(content['content_metadata']['panda_id'])
+                            content['content_metadata']['panda_id'] = panda_id
+                            content['substatus'] = panda_status
+                        elif content['content_metadata']['panda_id'] > panda_id:
+                            if 'old_panda_id' not in content['content_metadata']:
+                                content['content_metadata']['old_panda_id'] = []
+                            if panda_id not in content['content_metadata']['old_panda_id']:
+                                content['content_metadata']['old_panda_id'].append(panda_id)
+                            # content['content_metadata']['panda_id'] = content['content_metadata']['panda_id']
+                            # content['substatus'] = panda_status
+                        else:
+                            pass
+                            # content['content_metadata']['panda_id'] = panda_id
+                            content['substatus'] = panda_status
+                    else:
+                        content['content_metadata']['panda_id'] = panda_id
+                        content['substatus'] = panda_status
+
+                    update_contents.append(content)
+                    num_updated_contents += 1
+                else:
+                    num_unupdated_contents += 1
+        self.logger.debug("get_update_contents, num_updated_contents: %s, num_unupdated_contents: %s" % (num_updated_contents, num_unupdated_contents))
+        self.logger.debug("get_update_contents, update_contents[:5]: %s" % (str(update_contents[:5])))
+        return update_contents
+
+    def poll_panda_task(self, processing=None, input_output_maps=None):
+        task_id = None
+        try:
+            from pandaclient import Client
+
+            if processing:
+                proc = processing['processing_metadata']['processing']
+                task_id = proc.workload_id
+                if task_id is None:
+                    task_id = self.get_panda_task_id(processing)
+
+                if task_id:
+                    # ret_ids = Client.getPandaIDsWithTaskID(task_id, verbose=False)
+                    self.logger.debug("poll_panda_task, task_id: %s" % str(task_id))
+                    task_info = Client.getJediTaskDetails({'jediTaskID': task_id}, True, True, verbose=False)
+                    self.logger.debug("poll_panda_task, task_info[0]: %s" % str(task_info[0]))
+                    if task_info[0] != 0:
+                        self.logger.warn("poll_panda_task %s, error getting task status, task_info: %s" % (task_id, str(task_info)))
+                        return ProcessingStatus.Submitting, []
+
+                    task_info = task_info[1]
+
+                    processing_status = self.get_processing_status_from_panda_status(task_info["status"])
+
+                    if processing_status in [ProcessingStatus.SubFinished]:
+                        if self.retry_number < self.num_retries:
+                            self.reactivate_processing(processing)
+                            processing_status = ProcessingStatus.Submitted
+                            self.retry_number += 1
+
+                    all_jobs_ids = task_info['PandaID']
+
+                    terminated_jobs, inputname_mapid_map = self.get_job_maps(input_output_maps)
+                    self.logger.debug("poll_panda_task, task_id: %s, all jobs: %s, terminated_jobs: %s" % (str(task_id), len(all_jobs_ids), len(terminated_jobs)))
+
+                    all_jobs_ids = set(all_jobs_ids)
+                    terminated_jobs = set(terminated_jobs)
+                    unterminated_jobs = all_jobs_ids - terminated_jobs
+
+                    inputname_jobid_map = self.poll_panda_jobs(unterminated_jobs)
+                    intersection_keys = set(inputname_mapid_map.keys()) & set(inputname_jobid_map.keys())
+
+                    updated_contents = self.get_update_contents(list(intersection_keys), inputname_mapid_map, inputname_jobid_map)
+
+                    final_update_contents = []
+                    if processing_status in [ProcessingStatus.SubFinished, ProcessingStatus.Finished, ProcessingStatus.Failed]:
+                        if updated_contents:
+                            # there are still polling contents, should not terminate the task.
+                            log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still updated_contents[:5]: %s" % (processing['processing_id'],
+                                                                                                                                            task_id,
+                                                                                                                                            processing_status,
+                                                                                                                                            str(updated_contents[:5]))
+                            log_warn = log_warn + ". Keep the processing status as running now."
+                            self.logger.warn(log_warn)
+                            processing_status = ProcessingStatus.Running
+                        elif list(unterminated_jobs):
+                            log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still unterminated_jobs[:5]: %s" % (processing['processing_id'],
+                                                                                                                                             task_id,
+                                                                                                                                             processing_status,
+                                                                                                                                             str(list(unterminated_jobs)[:5]))
+                            log_warn = log_warn + ". Keep the processing status as running now."
+                            self.logger.warn(log_warn)
+                            processing_status = ProcessingStatus.Running
+                        else:
+                            # unsubmitted_inputnames = set(inputname_mapid_map.keys()) - set(inputname_jobid_map.keys())
+                            # unsubmitted_inputnames = list(unsubmitted_inputnames)
+                            # if unsubmitted_inputnames:
+                            #     log_warn = "Processing (%s) with panda task id (%s) is %s, however there are still unsubmitted_inputnames[:5]: %s" % (processing['processing_id'],
+                            #                                                                                                                           task_id,
+                            #                                                                                                                           processing_status,
+                            #                                                                                                                           str(unsubmitted_inputnames[:5]))
+                            #     log_warn = log_warn + ". Keep the processing status as running now."
+                            #     self.logger.warn(log_warn)
+                            #     processing_status = ProcessingStatus.Running
+
+                            for inputname in inputname_mapid_map:
+                                map_id_contents = inputname_mapid_map[inputname]
+                                contents = map_id_contents['outputs']
+                                for content in contents:
+                                    if (content['substatus'] not in [ContentStatus.Available, ContentStatus.FakeAvailable, ContentStatus.FinalFailed]):
+                                        content['content_metadata']['old_final_status'] = content['substatus']
+                                        content['substatus'] = ContentStatus.FinalFailed
+                                        # final_update_contents.append(content)     # TODO: mark other contents to Missing
+
+                            if final_update_contents:
+                                processing_status = ProcessingStatus.Running
+                    return processing_status, updated_contents + final_update_contents
+                else:
+                    return ProcessingStatus.Failed, []
+        except Exception as ex:
+            msg = "Failed to check the processing (%s) status: %s" % (str(processing['processing_id']), str(ex))
+            self.logger.error(msg)
+            self.logger.error(ex)
+            self.logger.error(traceback.format_exc())
+            # raise exceptions.IDDSException(msg)
+        return ProcessingStatus.Submitting, []
+
     def kill_processing(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
                 # task_id = processing['processing_metadata']['task_id']
@@ -759,7 +984,7 @@ class DomaPanDAWork(Work):
     def kill_processing_force(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
                 # task_id = processing['processing_metadata']['task_id']
@@ -772,7 +997,7 @@ class DomaPanDAWork(Work):
     def reactivate_processing(self, processing):
         try:
             if processing:
-                from pandatools import Client
+                from pandaclient import Client
                 # task_id = processing['processing_metadata']['task_id']
                 proc = processing['processing_metadata']['processing']
                 task_id = proc.workload_id
@@ -794,7 +1019,7 @@ class DomaPanDAWork(Work):
         update_processing = {}
         reset_expired_at = False
         reactive_contents = []
-        # self.logger.debug("poll_processing_updates, input_output_maps: %s" % str(input_output_maps))
+        self.logger.debug("poll_processing_updates, input_output_maps.keys[:5]: %s" % str(list(input_output_maps.keys())[:5]))
 
         if processing:
             proc = processing['processing_metadata']['processing']
@@ -836,7 +1061,7 @@ class DomaPanDAWork(Work):
 
             processing_status, poll_updated_contents = self.poll_panda_task(processing=processing, input_output_maps=input_output_maps)
             self.logger.debug("poll_processing_updates, processing_status: %s" % str(processing_status))
-            self.logger.debug("poll_processing_updates, update_contents: %s" % str(poll_updated_contents))
+            self.logger.debug("poll_processing_updates, update_contents[:10]: %s" % str(poll_updated_contents[:10]))
 
             if poll_updated_contents:
                 proc.has_new_updates()
@@ -890,10 +1115,10 @@ class DomaPanDAWork(Work):
 
         self.logger.debug("poll_processing_updates, task: %s, update_processing: %s" %
                           (proc.workload_id, str(update_processing)))
-        self.logger.debug("poll_processing_updates, task: %s, updated_contents: %s" %
-                          (proc.workload_id, str(updated_contents)))
-        self.logger.debug("poll_processing_updates, task: %s, reactive_contents: %s" %
-                          (proc.workload_id, str(reactive_contents)))
+        self.logger.debug("poll_processing_updates, task: %s, updated_contents[:100]: %s" %
+                          (proc.workload_id, str(updated_contents[:100])))
+        self.logger.debug("poll_processing_updates, task: %s, reactive_contents[:100]: %s" %
+                          (proc.workload_id, str(reactive_contents[:100])))
         return update_processing, updated_contents + reactive_contents, {}
 
     def get_status_statistics(self, registered_input_output_maps):
